var {
  _optionalChain
} = require('@sentry/utils');

Object.defineProperty(exports, '__esModule', { value: true });

const api = require('@opentelemetry/api');
const instrumentation = require('@opentelemetry/instrumentation');
const instrumentationHttp = require('@opentelemetry/instrumentation-http');
const core = require('@sentry/core');
const opentelemetry = require('@sentry/opentelemetry');
const scope = require('../sdk/scope.js');
const addOriginToSpan = require('../utils/addOriginToSpan.js');
const getRequestUrl = require('../utils/getRequestUrl.js');

const _httpIntegration = ((options = {}) => {
  const _breadcrumbs = typeof options.breadcrumbs === 'undefined' ? true : options.breadcrumbs;
  const _ignoreOutgoingRequests = options.ignoreOutgoingRequests;
  const _ignoreIncomingRequests = options.ignoreIncomingRequests;

  return {
    name: 'Http',
    setupOnce() {
      const instrumentations = [
        new instrumentationHttp.HttpInstrumentation({
          ignoreOutgoingRequestHook: request => {
            const url = getRequestUrl.getRequestUrl(request);

            if (!url) {
              return false;
            }

            if (core.isSentryRequestUrl(url, opentelemetry.getClient())) {
              return true;
            }

            if (_ignoreOutgoingRequests && _ignoreOutgoingRequests(url)) {
              return true;
            }

            return false;
          },

          ignoreIncomingRequestHook: request => {
            const url = getRequestUrl.getRequestUrl(request);

            const method = _optionalChain([request, 'access', _ => _.method, 'optionalAccess', _2 => _2.toUpperCase, 'call', _3 => _3()]);
            // We do not capture OPTIONS/HEAD requests as transactions
            if (method === 'OPTIONS' || method === 'HEAD') {
              return true;
            }

            if (_ignoreIncomingRequests && _ignoreIncomingRequests(url)) {
              return true;
            }

            return false;
          },

          requireParentforOutgoingSpans: true,
          requireParentforIncomingSpans: false,
          requestHook: (span, req) => {
            _updateSpan(span, req);

            // Update the isolation scope, isolate this request
            if (opentelemetry.getSpanKind(span) === api.SpanKind.SERVER) {
              const isolationScope = core.getIsolationScope().clone();
              isolationScope.setSDKProcessingMetadata({ request: req });

              const client = opentelemetry.getClient();
              if (client && client.getOptions().autoSessionTracking) {
                isolationScope.setRequestSession({ status: 'ok' });
              }
              scope.setIsolationScope(isolationScope);
            }
          },
          responseHook: (span, res) => {
            if (_breadcrumbs) {
              _addRequestBreadcrumb(span, res);
            }

            const client = opentelemetry.getClient();
            if (client && client.getOptions().autoSessionTracking) {
              setImmediate(() => {
                client['_captureRequestSession']();
              });
            }
          },
        }),
      ];

      instrumentation.registerInstrumentations({
        instrumentations,
      });
    },
  };
}) ;

const httpIntegration = core.defineIntegration(_httpIntegration);

/** Update the span with data we need. */
function _updateSpan(span, request) {
  addOriginToSpan.addOriginToSpan(span, 'auto.http.otel.http');

  if (opentelemetry.getSpanKind(span) === api.SpanKind.SERVER) {
    opentelemetry.setSpanMetadata(span, { request: request  });
  }
}

/** Add a breadcrumb for outgoing requests. */
function _addRequestBreadcrumb(span, response) {
  if (opentelemetry.getSpanKind(span) !== api.SpanKind.CLIENT) {
    return;
  }

  const data = opentelemetry._INTERNAL.getRequestSpanData(span);
  core.addBreadcrumb(
    {
      category: 'http',
      data: {
        status_code: response.statusCode,
        ...data,
      },
      type: 'http',
    },
    {
      event: 'response',
      // TODO FN: Do we need access to `request` here?
      // If we do, we'll have to use the `applyCustomAttributesOnSpan` hook instead,
      // but this has worse context semantics than request/responseHook.
      response,
    },
  );
}

exports.httpIntegration = httpIntegration;
//# sourceMappingURL=http.js.map
