Object.defineProperty(exports, '__esModule', { value: true });

const api = require('@opentelemetry/api');
const instrumentation = require('@opentelemetry/instrumentation');
const core = require('@sentry/core');
const opentelemetry = require('@sentry/opentelemetry');
const nodeVersion = require('../nodeVersion.js');
const addOriginToSpan = require('../utils/addOriginToSpan.js');

const _nativeNodeFetchIntegration = ((options = {}) => {
  const _breadcrumbs = typeof options.breadcrumbs === 'undefined' ? true : options.breadcrumbs;
  const _ignoreOutgoingRequests = options.ignoreOutgoingRequests;

  function getInstrumentation() {
    // Only add NodeFetch if Node >= 16, as previous versions do not support it
    if (nodeVersion.NODE_MAJOR < 16) {
      return;
    }

    try {
      // eslint-disable-next-line @typescript-eslint/no-var-requires
      const { FetchInstrumentation } = require('opentelemetry-instrumentation-fetch-node');
      return [
        new FetchInstrumentation({
          ignoreRequestHook: (request) => {
            const url = request.origin;
            return _ignoreOutgoingRequests && url && _ignoreOutgoingRequests(url);
          },

          onRequest: ({ span }) => {
            _updateSpan(span);

            if (_breadcrumbs) {
              _addRequestBreadcrumb(span);
            }
          },
        }),
      ];
    } catch (error) {
      // Could not load instrumentation
    }
  }

  return {
    name: 'NodeFetch',
    setupOnce() {
      const instrumentations = getInstrumentation();

      if (instrumentations) {
        instrumentation.registerInstrumentations({
          instrumentations,
        });
      }
    },
  };
}) ;

const nativeNodeFetchIntegration = core.defineIntegration(_nativeNodeFetchIntegration);

/** Update the span with data we need. */
function _updateSpan(span) {
  addOriginToSpan.addOriginToSpan(span, 'auto.http.otel.node_fetch');
}

/** Add a breadcrumb for outgoing requests. */
function _addRequestBreadcrumb(span) {
  if (opentelemetry.getSpanKind(span) !== api.SpanKind.CLIENT) {
    return;
  }

  const data = opentelemetry._INTERNAL.getRequestSpanData(span);
  core.addBreadcrumb({
    category: 'http',
    data: {
      ...data,
    },
    type: 'http',
  });
}

exports.nativeNodeFetchIntegration = nativeNodeFetchIntegration;
//# sourceMappingURL=node-fetch.js.map
