Object.defineProperty(exports, '__esModule', { value: true });

const api$1 = require('@opentelemetry/api');
const resources = require('@opentelemetry/resources');
const sdkTraceBase = require('@opentelemetry/sdk-trace-base');
const semanticConventions = require('@opentelemetry/semantic-conventions');
const core = require('@sentry/core');
const opentelemetry = require('@sentry/opentelemetry');
const utils = require('@sentry/utils');
const debugBuild = require('../debug-build.js');
const contextManager = require('../otel/contextManager.js');
const api = require('./api.js');

/**
 * Initialize OpenTelemetry for Node.
 */
function initOtel() {
  const client = api.getClient();

  if (!client) {
    debugBuild.DEBUG_BUILD &&
      utils.logger.warn(
        'No client available, skipping OpenTelemetry setup. This probably means that `Sentry.init()` was not called before `initOtel()`.',
      );
    return;
  }

  if (client.getOptions().debug) {
    const otelLogger = new Proxy(utils.logger , {
      get(target, prop, receiver) {
        const actualProp = prop === 'verbose' ? 'debug' : prop;
        return Reflect.get(target, actualProp, receiver);
      },
    });

    api$1.diag.setLogger(otelLogger, api$1.DiagLogLevel.DEBUG);
  }

  opentelemetry.setupEventContextTrace(client);

  const provider = setupOtel(client);
  client.traceProvider = provider;
}

/** Just exported for tests. */
function setupOtel(client) {
  // Create and configure NodeTracerProvider
  const provider = new sdkTraceBase.BasicTracerProvider({
    sampler: new opentelemetry.SentrySampler(client),
    resource: new resources.Resource({
      [semanticConventions.SemanticResourceAttributes.SERVICE_NAME]: 'node-experimental',
      [semanticConventions.SemanticResourceAttributes.SERVICE_NAMESPACE]: 'sentry',
      [semanticConventions.SemanticResourceAttributes.SERVICE_VERSION]: core.SDK_VERSION,
    }),
    forceFlushTimeoutMillis: 500,
  });
  provider.addSpanProcessor(new opentelemetry.SentrySpanProcessor());

  // Initialize the provider
  provider.register({
    propagator: new opentelemetry.SentryPropagator(),
    contextManager: new contextManager.SentryContextManager(),
  });

  return provider;
}

exports.initOtel = initOtel;
exports.setupOtel = setupOtel;
//# sourceMappingURL=initOtel.js.map
