import { _optionalChain } from '@sentry/utils';
import { SpanKind } from '@opentelemetry/api';
import { registerInstrumentations } from '@opentelemetry/instrumentation';
import { HttpInstrumentation } from '@opentelemetry/instrumentation-http';
import { defineIntegration, isSentryRequestUrl, getIsolationScope, addBreadcrumb } from '@sentry/core';
import { getClient, getSpanKind, setSpanMetadata, _INTERNAL } from '@sentry/opentelemetry';
import { setIsolationScope } from '../sdk/scope.js';
import { addOriginToSpan } from '../utils/addOriginToSpan.js';
import { getRequestUrl } from '../utils/getRequestUrl.js';

const _httpIntegration = ((options = {}) => {
  const _breadcrumbs = typeof options.breadcrumbs === 'undefined' ? true : options.breadcrumbs;
  const _ignoreOutgoingRequests = options.ignoreOutgoingRequests;
  const _ignoreIncomingRequests = options.ignoreIncomingRequests;

  return {
    name: 'Http',
    setupOnce() {
      const instrumentations = [
        new HttpInstrumentation({
          ignoreOutgoingRequestHook: request => {
            const url = getRequestUrl(request);

            if (!url) {
              return false;
            }

            if (isSentryRequestUrl(url, getClient())) {
              return true;
            }

            if (_ignoreOutgoingRequests && _ignoreOutgoingRequests(url)) {
              return true;
            }

            return false;
          },

          ignoreIncomingRequestHook: request => {
            const url = getRequestUrl(request);

            const method = _optionalChain([request, 'access', _ => _.method, 'optionalAccess', _2 => _2.toUpperCase, 'call', _3 => _3()]);
            // We do not capture OPTIONS/HEAD requests as transactions
            if (method === 'OPTIONS' || method === 'HEAD') {
              return true;
            }

            if (_ignoreIncomingRequests && _ignoreIncomingRequests(url)) {
              return true;
            }

            return false;
          },

          requireParentforOutgoingSpans: true,
          requireParentforIncomingSpans: false,
          requestHook: (span, req) => {
            _updateSpan(span, req);

            // Update the isolation scope, isolate this request
            if (getSpanKind(span) === SpanKind.SERVER) {
              const isolationScope = getIsolationScope().clone();
              isolationScope.setSDKProcessingMetadata({ request: req });

              const client = getClient();
              if (client && client.getOptions().autoSessionTracking) {
                isolationScope.setRequestSession({ status: 'ok' });
              }
              setIsolationScope(isolationScope);
            }
          },
          responseHook: (span, res) => {
            if (_breadcrumbs) {
              _addRequestBreadcrumb(span, res);
            }

            const client = getClient();
            if (client && client.getOptions().autoSessionTracking) {
              setImmediate(() => {
                client['_captureRequestSession']();
              });
            }
          },
        }),
      ];

      registerInstrumentations({
        instrumentations,
      });
    },
  };
}) ;

const httpIntegration = defineIntegration(_httpIntegration);

/** Update the span with data we need. */
function _updateSpan(span, request) {
  addOriginToSpan(span, 'auto.http.otel.http');

  if (getSpanKind(span) === SpanKind.SERVER) {
    setSpanMetadata(span, { request: request  });
  }
}

/** Add a breadcrumb for outgoing requests. */
function _addRequestBreadcrumb(span, response) {
  if (getSpanKind(span) !== SpanKind.CLIENT) {
    return;
  }

  const data = _INTERNAL.getRequestSpanData(span);
  addBreadcrumb(
    {
      category: 'http',
      data: {
        status_code: response.statusCode,
        ...data,
      },
      type: 'http',
    },
    {
      event: 'response',
      // TODO FN: Do we need access to `request` here?
      // If we do, we'll have to use the `applyCustomAttributesOnSpan` hook instead,
      // but this has worse context semantics than request/responseHook.
      response,
    },
  );
}

export { httpIntegration };
//# sourceMappingURL=http.js.map
