import { SpanKind } from '@opentelemetry/api';
import { registerInstrumentations } from '@opentelemetry/instrumentation';
import { defineIntegration, addBreadcrumb } from '@sentry/core';
import { getSpanKind, _INTERNAL } from '@sentry/opentelemetry';
import { NODE_MAJOR } from '../nodeVersion.js';
import { addOriginToSpan } from '../utils/addOriginToSpan.js';

const _nativeNodeFetchIntegration = ((options = {}) => {
  const _breadcrumbs = typeof options.breadcrumbs === 'undefined' ? true : options.breadcrumbs;
  const _ignoreOutgoingRequests = options.ignoreOutgoingRequests;

  function getInstrumentation() {
    // Only add NodeFetch if Node >= 16, as previous versions do not support it
    if (NODE_MAJOR < 16) {
      return;
    }

    try {
      // eslint-disable-next-line @typescript-eslint/no-var-requires
      const { FetchInstrumentation } = require('opentelemetry-instrumentation-fetch-node');
      return [
        new FetchInstrumentation({
          ignoreRequestHook: (request) => {
            const url = request.origin;
            return _ignoreOutgoingRequests && url && _ignoreOutgoingRequests(url);
          },

          onRequest: ({ span }) => {
            _updateSpan(span);

            if (_breadcrumbs) {
              _addRequestBreadcrumb(span);
            }
          },
        }),
      ];
    } catch (error) {
      // Could not load instrumentation
    }
  }

  return {
    name: 'NodeFetch',
    setupOnce() {
      const instrumentations = getInstrumentation();

      if (instrumentations) {
        registerInstrumentations({
          instrumentations,
        });
      }
    },
  };
}) ;

const nativeNodeFetchIntegration = defineIntegration(_nativeNodeFetchIntegration);

/** Update the span with data we need. */
function _updateSpan(span) {
  addOriginToSpan(span, 'auto.http.otel.node_fetch');
}

/** Add a breadcrumb for outgoing requests. */
function _addRequestBreadcrumb(span) {
  if (getSpanKind(span) !== SpanKind.CLIENT) {
    return;
  }

  const data = _INTERNAL.getRequestSpanData(span);
  addBreadcrumb({
    category: 'http',
    data: {
      ...data,
    },
    type: 'http',
  });
}

export { nativeNodeFetchIntegration };
//# sourceMappingURL=node-fetch.js.map
